import 'package:equatable/equatable.dart';
import 'package:json_annotation/json_annotation.dart';
import 'package:open_library/models/ol_search_doc_model.dart';

part 'ol_search_model.g.dart';

abstract class OLSearchBase extends Equatable {
  @override
  List<Object?> get props => [];

  const OLSearchBase();
}

class OLSearchError extends OLSearchBase {
  static const message = "Error on returning valid Search Result Objects";

  @override
  List<Object?> get props => [message];

  @override
  bool? get stringify => false;

  const OLSearchError();

  @override
  String toString() {
    return 'Error: $message';
  }
}

@JsonSerializable()
class OLSearch extends OLSearchBase {
  final int numFound;
  final bool numFoundExact;
  final String q;
  final List<OLSearchDoc> docs;

  @JsonKey(ignore: true)
  final String patterns;

  @override
  List<Object?> get props => [numFound, numFoundExact, q, patterns];

  const OLSearch({
    this.numFound = 0,
    this.numFoundExact = false,
    this.q = "",
    this.docs = const [],
    this.patterns = "",
  });

  factory OLSearch.fromJson(Map<String, dynamic> json) {
    return _$OLSearchFromJson(json);
  }

  Map<String, dynamic> toJson() {
    return _$OLSearchToJson(this);
  }

  @override
  String toString() {
    return '\nSearch:\n  numFound:$numFound\n  numFoundExact:$numFoundExact\n  q:$q\n  numDocs:${docs.length}\n  patterns:$patterns';
  }
}
